/* eslint-disable no-console */
import {
  Select, Card, Text, CheckboxMultiple, Textarea, Button, AlertService, LoadingService, Form,
} from '@zeedhi/common';
import {
  IDictionary, Http, Metadata, IEventParam, Router, Loader, SimpleDatasource,
} from '@zeedhi/core';
import {
  IterableComponentRender, IterableCarousel,
} from '@zeedhi/teknisa-components-common';
import { Client } from '@pusher/push-notifications-web';
import { Utilities } from '@/util/Utilities';
import { getRouteParams } from '../router';

export class ListagemVistoriasController {
  public hierarquiasController = Loader.getInstance('HierarquiasController');

  // Controlla a visibilidade dos elementos de acordo com a permissão do usuário
  public isDisabledElement = (localStorage.getItem('idpermisssvistoria') === 'V');

  public quantidadeAtividades: Number = 0;

  public trueFalseValue: String = 'Não';

  public idTarefa: IDictionary = {
    IDSITUACAOAGEND: [],
    IDSITUACAOGERCHK: [],
  };

  public selected: IDictionary = {};

  public dataBase: IDictionary = [];

  public dataSelects: IDictionary = {
    selectPlace: [],
    selectEmplyoee: [],
    selectSituation: [],
  };

  private dataFilter: IDictionary = {
    selectPlace: '',
    selectSituation: '',
    selectEmplyoee: '',
    'TO_CHAR(GCHK.DTHORAAGEND,\'DD/MM/YYYY\')': '',
  };

  public setData(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public filter() {
    this.dataFilter.selectPlace = this.getPlace;
    this.dataFilter.selectSituation = this.getSituation;
    this.dataFilter.selectEmplyoee = this.getEmployee;
    this.dataFilter['TO_CHAR(GCHK.DTHORAAGEND,\'DD/MM/YYYY\')'] = (this.getDate != null) ? `TO_CHAR('${this.getDate}')` : null;
    this.createCardSurvey();
  }

  get getPlace() { // pegar o select Local
    return Metadata.getInstance<Select>('selectPlace').value || '';
  }

  get getSituation() { // pegar o select Situação/Status
    return Metadata.getInstance<Select>('selectSituation').value || '';
  }

  get getEmployee() { // pegar o select Situação/Status
    return Metadata.getInstance<Select>('selectEmplyoee').value || '';
  }

  get getDate() { // pegar o select Situação/Status
    return Metadata.getInstance<Select>('selectDate').value || '';
  }

  public refreshData({ component }: { component: Select }) {
    component.datasource.get();
  }

  public setPlace({ component }: any) { // trazer os valores do banco e colocar no select local
    Http.get('/getPlaceSurvey').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.place);
    });
  }

  public setSituation({ component }: any) { // trazer os valores do banco e colocar no select situação
    Http.get('/getSituationSurvey').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.situation);
    });
  }

  public getDataInfo({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  public async createCardSurvey() { // Criar os cards da lista de vistorias
    LoadingService.show();
    this.hierarquiasController.mountHierarchyUsersArray().then((retorno: IDictionary) => {
      Metadata.getInstance<IterableComponentRender>('iCardSurveyLocations').datasource.data = [];
      Http.post('/getCardsSurveys', { filter: this.dataFilter, usersIds: retorno }).then((response: IDictionary) => {
        response.data.dataset.cardsSurveys.forEach((dados: IDictionary) => {
          const DateTime = dados.DTHORAAGEND.split(' ');
          Metadata.getInstance<IterableComponentRender>('iCardSurveyLocations').datasource.data.push({
            NRTAREFA: dados.NRGERENCIAMENTOCHK,
            DTHORA: DateTime[1],
            DATAH: DateTime[0],
            NMLOCAL: dados.NMGERENCIAMENTOCHK,
            IDSITUACAO: dados.IDSITUACAOGERCHK,
            NMTIPOLOCAL: `${dados.NMLOCALCLIENTE} | ${dados.NMLOCALPAI}`,
            NMFUNCIONARIO: dados.NMOPERADOR,
          });
        });
      });
    }).finally(() => {
      LoadingService.hide();
    }); // Busca os usuários subordinados do usuário logado
  }

  // -------------> FUNÇÕES DA TELA DOS CRITÉRIOS <----------------
  public showCriteria({ component }:{ component: Card }) {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    if (Object.keys(this.selected).length) {
      this.dataBase[this.selected.name] = {
        NRGERENCIAMENTOCHK: nr,
        NRATIVIDADECHK: this.selected.name.substr(9),
        valuesChkbox: Metadata.getInstance<CheckboxMultiple>('checkbox-multiple').value.map((dados: any) => `${dados}`).join(','),
        valueObservation: Metadata.getInstance<Textarea>('textAreaObservation').value,
      };
      this.selected.cssClass = 'card3 cardAct cardAct2 card-disable';
    }
    component.cssClass = 'card-background card3 cardAct cardAct2';
    this.selected = component;
    const title: any = component!.children[0]!;
    const titleAux: any = title!.children[1]!;
    const header: any = titleAux!.children[0]!;

    Metadata.getInstance<Text>('subTitleHeader').text = header.text;
    Metadata.getInstance<Text>('textSelectConformities').text = 'Marque os campos de não conformidades:';
    Metadata.getInstance<Textarea>('textAreaObservation').isVisible = true;

    Http.post('/getTask', {
      NRATIVIDADECHK: component.name.substr(9),
      NRGERENCIAMENTOCHK: nr,
    }).then((response: IDictionary) => {
      response.data.dataset.criterias.forEach((criterias: IDictionary) => {
        this.idTarefa.IDSITUACAOGERCHK = criterias.IDSITUACAOGERCHK;
        this.idTarefa.IDSITUACAOAGEND = criterias.IDSITUACAOAGEND;
        const validaObs = criterias.IDOBRIGAOBSERVACAO;

        if (criterias.IDSITUACAOGERCHK === '3' || criterias.IDSITUACAOAGEND !== '4') {
          this.activateButtonFinalize();// desativa o botão finalize, torna apenas visualização
        } else {
          this.activateButton();
        }
        if (validaObs !== 'S') {
          Metadata.getInstance<Textarea>('textAreaObservation').clearValidations();
        } else {
          Metadata.getInstance<Textarea>('textAreaObservation').addValidation('required', {});
        }
        const chkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-multiple');
        if (chkbox.datasource instanceof SimpleDatasource) {
          chkbox!.datasource!.items = criterias.CRITERIOS;
        }
        if (Object.keys(this.dataBase).includes(component.name)) {
          chkbox.value = this.dataBase[component.name].valuesChkbox.replaceAll('\'').split(',');
          Metadata.getInstance<Textarea>('textAreaObservation').value = this.dataBase[component.name].valueObservation;
        } else {
          chkbox.value = criterias.CRITERIOSATIVOS;
          Metadata.getInstance<Textarea>('textAreaObservation').value = criterias.DSOBSERVACAO;
        }
      });
    });
  }

  public activateButton() {
    const saveButton = Metadata.getInstance<Button>('saveButton');

    saveButton.disabled = false;
  }

  public showDetailsSurvey() {
    const routeParams = getRouteParams();
    const { nr, local } = routeParams;
    Metadata.getInstance<Text>('textTitleLocal').text = local;
    Http.post('/showDetailsSurvey', { nr }).then((response: IDictionary) => {
      this.quantidadeAtividades = response.data.dataset.detailsSurvey.length;
      response.data.dataset.detailsSurvey.forEach((dados: IDictionary) => {
        Metadata.getInstance<IterableCarousel>('iCarousel').iterable.datasource.data.push({
          NRTAREFA: dados.NRATIVIDADECHK,
          NAME: dados.NMATIVIDADE,
        });
      });
    });
  }

  public saveCriterias({ component }:{ component:Button }) {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    let id: any;
    let message: any;
    const formObs = Metadata.getInstance<Form>('Form-observation');

    this.dataBase[this.selected.name] = {
      NRGERENCIAMENTOCHK: nr,
      NRATIVIDADECHK: this.selected.name.substr(9),
      valuesChkbox: Metadata.getInstance<CheckboxMultiple>('checkbox-multiple').value.map((dados: any) => `'${dados}'`).join(','),
      valueObservation: Metadata.getInstance<Textarea>('textAreaObservation').value,
    };
    const dataBase = Object.keys(this.dataBase).map((key: string) => this.dataBase[key]);

    const selectedCriteria = Metadata.getInstance<CheckboxMultiple>('checkbox-multiple').value.map((dados: any) => `${dados}`);
    // console.log(selectedCriteria);
    if (component.name === 'saveButton') {
      id = 2;
      message = 'Atividade salva com sucesso!';
      const areaComentatio = Metadata.getInstance<Textarea>('textAreaObservation');
      if (Object.keys(selectedCriteria).length === 0) {
        console.log(Object.keys(selectedCriteria).length);
        if (areaComentatio.value === '') {
          Metadata.getInstance<Textarea>('textAreaObservation').value = 'Vistoria realizada, sem não-conformidades';
        }
      }
      if (Object.keys(this.dataBase).length === this.quantidadeAtividades) {
        Metadata.getInstance<Button>('finalizeButton').disabled = false;
      }
    } else {
      id = 3;
      message = 'Vistoria realizada com sucesso!';
      this.activateButtonFinalize();
    }

    if (Utilities.validateForm(formObs)) {
      LoadingService.show();
      Http.post('/upSurveyCriterias', [id, dataBase]).then(() => {
        Utilities.alertMessage(message, 'sucess');
        if (id === 3) {
          Router.replace('/Vistorias');
        }
      }).catch((error: any) => {
        if (component.name !== 'saveButton') {
          Utilities.showErrorMessage(error);
        }
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  public activateButtonFinalize() {
    const chkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-multiple');
    const tArea = Metadata.getInstance<Textarea>('textAreaObservation');
    const saveButton = Metadata.getInstance<Button>('saveButton');
    chkbox.readonly = true;
    tArea.readonly = true;
    saveButton.disabled = true;
  }

  public stylizeStatus({ component }: IEventParam<Text>) {
    if (component.text === 'Aguardando Vistoria') {
      component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;'>AGUARDANDO VISTORIA</span>";
    }
    if (component.text === 'Avaliado') {
      component.text = "<span  style='background-color: #dee2e6; color: #2b9348; font-size: 12px;'>A V A L I A D O</span>";
    }
    if (component.text === 'Em andamento') {
      component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 12px;'>EM ANDAMENTO</span>";
    }
    if (component.text === 'Cancelado') {
      component.text = "<span  style='background-color:#dee2e6; color: black; font-size: 12px;'>C A N C E L A D O</span>";
    }
    if (component.text === 'Tarefa Pendente') {
      component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;'>TAREFA PENDENTE</span>";
    }
  }

  public subscribeSurveyNotifications() {
    const idInst: string = process.env.VUE_APP_PUSH_NOTIFICATION_INSTANCE_ID!;
    const beamsClient = new Client({
      instanceId: idInst,
    });
    Notification.requestPermission()
      .then((result) => {
        if (result === 'granted') {
          beamsClient.start()
            .then(() => {
              const nrUser = window.localStorage.getItem('UserLogado');
              const devInterest = `survey-${nrUser}`;
              beamsClient.addDeviceInterest(devInterest)
                .then(() => {
                  AlertService.show({
                    name: 'basic-alert',
                    text: 'Agora você receberá notificações sobre suas vistorias',
                    top: true,
                    left: false,
                    timeout: 3000,
                    color: 'success',
                  });
                })
                .catch((err : any) => {
                  AlertService.show({
                    name: 'basic-alert',
                    text: err,
                    top: true,
                    left: false,
                    timeout: 3000,
                    color: 'error',
                  });
                });
            })
            .catch((err : any) => {
              AlertService.show({
                name: 'basic-alert',
                text: err,
                top: true,
                left: false,
                timeout: 3000,
                color: 'error',
              });
            });
        } else {
          AlertService.show({
            name: 'basic-alert',
            text: 'Você não permitiu que as notificações sejam enviadas. Você pode tentar ativar novamente se mudar ideia.',
            top: true,
            left: false,
            timeout: 3000,
            color: 'warning',
          });
        }
      })
      .catch((err) => {
        AlertService.show({
          name: 'basic-alert',
          text: err,
          top: true,
          left: false,
          timeout: 3000,
          color: 'error',
        });
      });
  }
}
